﻿using EvaPOS_API_FRAME.RespuestasXML;
using EvaPosSrvDTO;
using EvaPosSrvResp;
using GatewaySCO;
using RestSharp;
using SCOGateway.POSGk;
using Serilog;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using Microsoft.Extensions.Configuration;


namespace gatewayGK.ComandosGk
{
    public class InitializeRequestCmdGk : IComando
    {
        public string Referencia { get; set; } = "scsns:Initialize";
        /// <summary>
        /// DTO con solicitud.
        /// </summary>
        public InitializeRequestDTO Request { get; private set; }

        /// <summary>
        /// Url servicio autenticación Gk.
        /// </summary>
        public const string UrlLogin = ConstantesGK.UrlBaseMaquetaProd + "/com.gk_software.pos.api.service.session.PosSessionService/login";

        /// <summary>
        /// Procesa y responde el login.
        /// </summary>
        /// 

        public Respuestas Ejecutar()
        {
            // Crea una instancia de IConfiguration para acceder a la configuración
            IConfiguration configuration = new ConfigurationBuilder()
                .AddJsonFile("appsettings.json", optional: true, reloadOnChange: true)
                .Build();

            //Llamar valores del archivo de configuración
            string retailStoreId = configuration["DataGK:RetailStoreId"];
            string workstationId = configuration["DataGK:WorkstationId"];
            string workstationAddress = configuration["DataGK:WorkstationAddress"];
            string workstationTypeCode = configuration["DataGK:WorkstationTypeCode"];
            string loginName = configuration["DataGK:LoginName"];
            string businessUnitGroupID = configuration["DataGK:BusinessUnitGroupID"];
            string tillID = configuration["DataGK:TillID"];
            string password = configuration["DataGK:Password"];

            string jSessionId = configuration["DataGK:SessionID"]; ;
            string cookieValue = $"JSESSIONID={jSessionId}";

            Respuestas respuestaChec = null;
            int terminalNumber = Request.TerminalNumber;
            Log.Debug("Cmd InitializeRequestCmd ejecutado - Gk pruebas. Request id: {id}, terminal: {terminal}, recovery: {recovery}", Request.OperatorID, Request.TerminalNumber, Request.Recovery);
            // Contactar Smart POS Gk.

            var options = new RestClientOptions(UrlLogin)
            {
                MaxTimeout = -1,
                RemoteCertificateValidationCallback = (sender, certificate, chain, sslPolicyErrors) => true
            };
            
            var client = new RestClient(options);
            var request = new RestRequest(UrlLogin, Method.Post);
            request.AddHeader("Accept", "application/json;Format=GK-PLAIN-JSON");
            request.AddHeader("Content-Type", "application/json");
            request.AddHeader("Cookie", cookieValue);

            var datosAutenticacion = new AutenticationReq
            {
                RetailStoreId = retailStoreId,
                WorkstationId = workstationId,
                WorkstationAddress = workstationAddress,
                WorkstationTypeCode = workstationTypeCode,
                LoginName = loginName,
                Password = password,
                TrainingMode = false,
                FinalizeControlTransactionFlag = false,
                UseLoginTypeTechnicalForLoginManager = false,
                TillId = new Till
                {
                    TillDoKey = new TillDoKey
                    {
                        BusinessUnitGroupID = businessUnitGroupID,
                        TillID = tillID
                    }
                }
            };
          
            var loginRequestWrapper = new LoginRequestWrapper
            {
                LoginRequest = datosAutenticacion
            };

            string json = JsonSerializer.Serialize(loginRequestWrapper, new JsonSerializerOptions { WriteIndented = true });

            request.AddJsonBody(json);
            var response = client.Execute(request);

            if (response.IsSuccessful)
            {
                Log.Debug($"Gk: Autenticate - POST - response {response}");
                var autenticacionResp = JsonSerializer.Deserialize<AuthenticateResp>(response.Content);

                var transactionId = autenticacionResp.primaryEntry.transaction.key.transactionID;
                var operatorID = autenticacionResp.primaryEntry.transaction.operatorID;
                Log.Debug($"Transaction ID: {transactionId}, operatorID: {operatorID} ");
                // Salvar Transaccion y id de la session.
                Entorno<EntornoGK>.Instancia.getDatos().TransactionID = transactionId;
                Entorno<EntornoGK>.Instancia.getDatos().SeccionId = jSessionId;

                // Respuestas a SCO.
                var posbcStatus1 = new POSBCStatusEvent(1, TipoMensaje.Event, "INFO", "CONNECTING_TO_POS", "Connecting to POS system");
                var posbcStatus2 = new POSBCStatusEvent(1, TipoMensaje.Event, "INFO", "CONNECTED_TO_POS", "Connected to POS system");
                var posbcStatus3 = new POSBCStatusEvent(1, TipoMensaje.Event, "INFO", "POS_RESOURCES_INITIALIZED", "POS Resources Initialized");
                var inicializacionOk = new InitializeResponse(1, TipoMensaje.Resp, terminalNumber);
                respuestaChec = new Respuestas { posbcStatus1, posbcStatus2, posbcStatus3, inicializacionOk };
            }
            else
            {
                Log.Error($"Failed to connect to GK web service: {response.StatusCode}");
                Log.Error(response.Content);
                throw new ApplicationException("Servicio POST 'Login' en error.");
            }

            return respuestaChec;
        }

        public IComando CreaCopia()
        {
            return (InitializeRequestCmdGk)this.MemberwiseClone();
        }

        public void CargaDTO(DTOBase initializeRequestDTO)
        {
            Request = (InitializeRequestDTO)initializeRequestDTO;
        }
    }
}



