﻿using EvaPOS_API_FRAME.Comandos;
using EvaPOS_API_FRAME.DTO;
using EvaPOS_API_FRAME.RespuestasXML.AddCustomer;
using EvaPOS_API_FRAME.RespuestasXML;
using EvaPosSrvDTO;
using EvaPosSrvResp;
using Serilog;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SCOGateway.POSGk;
using RestSharp;
using gatewayGK.POSGk;

namespace gatewayGK.ComandosGkPruebas
{
    /// <summary>
    /// Comando para agregar un cliente
    /// </summary>
    public class AddCustomerCmdGKTest : IComando
    {
        public string Referencia { get; set; } = "scsns:AddCustomer";

        /// <summary>
        /// DTO con solicitud.
        /// </summary>
        public AddCustomerRequest Request { get; set; }
        /// <summary>
        /// Procesa y responde AddCustomerRequest.
        /// </summary>
        /// 
        public const string UrlCustomer = ConstantesGK.UrlBaseMaqueta + "/com.gk_software.pos.api.service.transaction.customer.CustomerTransactionService/registerCustomer";
        /// <summary>
        /// Url servicio autenticación Gk.
        /// </summary>
        public Respuestas Ejecutar()
        {
            Log.Debug("Cmd AddCustomerRequestCmd ejecutado. AddCustomerRequest : {AddCustomerRequest}", Request.RequestID);
            int requestId = Request.RequestID;
            Respuestas respuesta = null;

            var date = DateTime.Now.ToString("dd/MM/yyyy");
            CultureInfo culture = new CultureInfo("en-US");
            culture.DateTimeFormat.AMDesignator = "AM";
            culture.DateTimeFormat.PMDesignator = "PM";
            var time = DateTime.Now.ToString("hh:mm tt", culture);

            string idCliente = Request.CustomerIdentifier.CustomerID;
            // Contactar Smart POS Gk.
            var client = new RestClient(UrlCustomer);
            var request = new RestRequest(UrlCustomer, Method.Post);
            var datosCliente = new AddCustomerReq { Identifier = idCliente };
            request.AddJsonBody(datosCliente);

            // Ejecuta invocación sincrónica, deserializando respuesta según el tipo indicado en la invocación del Post<T>.
            var clienteResp = client.Post<AddCustomerResp>(request) ?? throw new ApplicationException("Servicio POST 'Cliente' retorna nulos.");
            Log.Debug($"GkPruebas: Respuesta Cliente - POST - response {clienteResp}");

            if( clienteResp != null)
            {
                var transactionStatusEvent = new TransactionStatusEvent(1, TipoMensaje.Event, requestId, "TRANSACTION_START", 1, "regularSale", "sales", date, time);
                var pOSReceiptEvent = new POSReceiptEvent(1, TipoMensaje.Event, requestId, "Customer", 0, "Body", requestId, 1, "center", "LineItem", "ItemSale", "Cliente:" + " " + "Brayan", "Add");
                var posReceiptEventHeader = new POSReceiptEventHeaderResponse(1, TipoMensaje.Event, requestId);
                var addCustomer = new AddCustomerResponse(1, TipoMensaje.Resp, requestId, "123456", "Brayan", 1234);
                respuesta = new Respuestas { transactionStatusEvent, posReceiptEventHeader, pOSReceiptEvent, addCustomer };
            }
            else
            {
                var addCustomerNotFound = new AddCustomerResponseNoEncontrado(1, TipoMensaje.Resp, requestId, "Usuario No encontrado", "INVALID_LOYALTY_NUMBER");
                respuesta = new Respuestas { addCustomerNotFound };
            }

            return respuesta;
        }

        public IComando CreaCopia()
        {
            return (AddCustomerCmd)this.MemberwiseClone();
        }
        public void CargaDTO(DTOBase addCustomerRequest)
        {
            Request = (AddCustomerRequest)addCustomerRequest;
        }
    }
}
