﻿using EvaPOS_API_FRAME.Comandos;
using EvaPOS_API_FRAME.DTO;
using EvaPOS_API_FRAME.RespuestasXML;
using EvaPosSrvDTO;
using EvaPosSrvResp;
using gatewayGK.POSGk;
using RestSharp;
using SCOGateway.POSGk;
using Serilog;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace gatewayGK.ComandosGkPruebas
{
    public class SubtotalCmdGKTest : IComando
    {
        public string Referencia { get; set; } = "scsns:GetTotals";
        /// <summary>
        /// DTO con solicitud.
        /// </summary>
        public GetTotalsRequest Request { get; private set; }

        /// <summary>
        /// Procesa y responde GetTotalsRequest.
        /// </summary>
        /// 
        public const string UrlSubtotal = ConstantesGK.UrlBaseMaqueta + "/com.gk_software.pos.api.service.subtotal.SubtotalService/processSubtotalCalculation";
        /// <summary>
        /// Url servicio subtotal Gk.
        /// </summary>

        public Respuestas Ejecutar()
        {

            Log.Debug("Cmd GetTotalsRequestDTO ejecutado. GetTotalsRequest : {GetTotalsRequest}", Request.RequestID);

            GetTotalsResponse getTotalsResponse = null;
            POSReceiptEventResponse pOSReceiptEvent = null;
            TotalsEventResponse totalEvent = null;
            AddItemResponseError addItemResponseError = null;
            int RequestId = Request.RequestID;
            Respuestas respuesta = null;

            var total = new RestClient(UrlSubtotal);
            var request = new RestRequest(UrlSubtotal, Method.Post);
            var datosTotal = new SubtotalReq { PricePositionLimit = 1000 };
            request.AddJsonBody(datosTotal);

            // Ejecuta invocación sincrónica, deserializando respuesta según el tipo indicado en la invocación del Post<T>.
            var subtotalResp = total.Post<SubTotalResp>(request) ?? throw new ApplicationException("Servicio POST 'Subtotal' retorna nulos.");
            Log.Debug($"GkPruebas: Respuesta Cliente - POST - response {subtotalResp}");

            pOSReceiptEvent = new POSReceiptEventResponse(1, TipoMensaje.Event, RequestId, RequestId, subtotalResp.ExtendedAmount.ToString(), subtotalResp.TotalAmount.ToString());
            totalEvent = new TotalsEventResponse(1, TipoMensaje.Event, RequestId, subtotalResp.TotalAmount.ToString(), subtotalResp.ExtendedAmount.ToString(), subtotalResp.TaxAmount.ToString(), subtotalResp.TotalAmount.ToString(), "0.00", "0.00", "0.00", "0.00", "0.00" , subtotalResp.Quantity, 0, "0.00", "0.00");
            getTotalsResponse = new GetTotalsResponse(1, TipoMensaje.Resp, RequestId, subtotalResp.TotalAmount.ToString(), subtotalResp.ExtendedAmount.ToString(), subtotalResp.TaxAmount.ToString(), subtotalResp.TotalAmount.ToString(), "0.00", "0.00", "0.00", "0.00", "0.00", subtotalResp.Quantity, 0, "0.00", "0.00");

            respuesta = new Respuestas { pOSReceiptEvent, totalEvent, getTotalsResponse };
            return respuesta;

        }

        public IComando CreaCopia()
        {
            return (GetTotalsRequestCmd)this.MemberwiseClone();
        }
        public void CargaDTO(DTOBase getTotalsRequestDTO)
        {
            Request = (GetTotalsRequest)getTotalsRequestDTO;
        }
    }
}
