﻿using Serilog;
using Microsoft.Extensions.Configuration;
using EvaPosSCOSrv;
using EvaPosSrvAplicacionImp;
using EvaPosSrvRespImp;

/// <summary>
/// EvaPos-API : servidor api, sockets y rest.
/// Usa Serilog para bitácora.
/// </summary> 
class Program
{
    static void Main(string[] args)
    {
        try
        {
            Console.WriteLine("*** Gateway SCO ***");
            // Instancia objeto de configuración, usa proveedor json y variables ambiente.
            IConfigurationRoot config = new ConfigurationBuilder()
                .AddJsonFile("appsettings.json")
                .AddEnvironmentVariables()
                .Build();

            // Lee archivo de configuración.
            Config? configuracion = config.GetRequiredSection("GatewayConfig").Get<Config>();
            if (configuracion is null)
            {
                throw new ApplicationException("Error en archivo de configuración, sección 'GatewayConfig' no encontrada.");
            }

            // TODO Sacar la ruta de logs del objeto de configuración adecuadamente.
            string? rutaLogs = config["Serilog:WriteTo:1:Args:path"];
            Console.WriteLine($"Archivo de logs en {rutaLogs}");

            // Instancia Serilog según parámetros en archivo de configuración.
            Log.Logger = new LoggerConfiguration()
                .ReadFrom.Configuration(config)
                .CreateLogger();
            Log.Information(configuracion.ToString());

            // TODO - opción de incluir la activación en la cadena de configuración.
            // TODO - aunque hace uso de una "invocación fluida" no parece ser el tipo de api que se beneficia de ese esquema.
            ServidorSocket servidor = new ServidorSocket()
                .ConIp(configuracion.IpSCO)
                .EnPuerto(configuracion.PortSCO)
                .AgregaDispensadorAdaptadores(IniciaDirectorioAdaptadores.Cargar().DirectorioAdaptadores)
                .AgregaDirectorioCmds(DirectorioCmdsFactory.CreaDirectorio(configuracion))
                .AgregaProcesadorAplicacion(new Aplicacion())
                .Activa();
        }
        catch (Exception e)
        {
            Log.Error($"Excepción: {e}");
            Environment.Exit(1);
        }
    }
}